#ifndef _bicoti_actor_graphic
#define _bicoti_actor_graphic

#include <source/actor/actor_basic.hpp>
#include <source/actor/font.hpp>
#include <source/interaction/interaction_mouse.hpp>
#include <source/interaction/interaction_mouse_motion.hpp>
#include <source/pixel/pixel.hpp>

//--------------------DECLARATIONS---------------------
//-----------------bicotiActorGraphic------------------

/**
 * Clase base para todos los actores grficos.
 *
 * Todos los actores grficos tienen la propiedad de tener una representacin grfica en pantalla. Son fuente de eventos de ratn y de movimiento de ratn.
 *@see bicotiActorGraphic2D 
 *@see bicotiActorGraphicMenu 
 *@see bicotiActorGraphicMenuBar 
 *@see bicotiActorGraphicMenuPopup 
 */
class bicotiActorGraphic : public bicotiActor, public bicotiInteractionSourceMouse, public bicotiInteractionSourceMouseMotion
{
protected:
  bicotiActorGraphic * ptr_parent;
  vector< bicotiActorGraphic * > progenies;

  bicotiPixelRGBA background_color;
  bicotiPixelRGBA foreground_color;

  bool is_visible;
  bool is_hidden;
  
  void Hide();
  void UnHide();

  bool is_tracking_on;
  bool main_actor;

  /**
   * Activa eventos de movimiento de ratn.
   *
   * Para uso interno. 
   *@see #DeactivateMouseMotionTracking
   */
  void ActivateMouseMotionTracking();
  
  /**
   * Desactiva eventos de movimiento de ratn.
   *
   * Para uso interno. Desactiva los eventos de movimientos de ratn. Esto permite mayor eficiencia al no tener que procesar dichos eventos cuando no se desea.
   *@see #ActivateMouseMotionTracking
   */
  void DeactivateMouseMotionTracking();

  bicotiFont current_font;

public:
  
  /**
   * Constructor
   * 
   * Construye un nuevo bicotiActorGraphic.
   *@param act_name nombre del actor.
   */
  bicotiActorGraphic( char * act_name = NULL );
  
  /** Destructor */
  virtual ~bicotiActorGraphic();
  
  /** 
   * Destruye el homlogo.
   *
   * Destruye el homlogo, liberando la memoria que este ocupaba.
   */
  void DestroyHomologous();

  /**
   * Agrega un hijo al actor
   *
   * Agrega el actor indicado a la lista de hijos del actor.
   *@param ptr_actor actor hijo.
   *@exception bicotiException si el actor hijo es NULL.
   */
  void SetChild( bicotiActorGraphic * ptr_actor); /* Poner child o progeny */
  
  /** 
   * Remueve un hijo del actor
   *
   * Quita al actor indicado de la lista de hijos del actor.
   *@param ptr_actor actor hijo
   *@see #SetChild
   */
  void RemoveChild( bicotiActorGraphic * );
  
  /** 
   * Devuelve un puntero al padre del actor.
   *
   *@returns un puntero al padre del actor, NULL si no tiene padre. 
   *@see #SetChild
   */ 
  bicotiActorGraphic * GetParent();
 
  /**
   * Devuelve un puntero a un hijo del actor.
   *
   * Devuelve un puntero al hijo del actor correspondiente al ndice.
   *@param index indice del actor en la lista de hijos.
   *@returns un puntero al actor. El resultado es indeterminado si index est fuera de rango.  
   *@see #CountProgenies
   */
  bicotiActorGraphic * GetProgeny( int index );
  
  /** 
   * Devuelve la cantidad de hijos del actor.
   *
   *@returns cantidad de hijos del actor
   */
  int CountProgenies();

  /**
   * Fija el color de fondo.
   *
   *@param color color de fondo.
   */
  void SetBackgroundColor( bicotiPixelRGBA color );
  
  /** 
   * Devuelve el color de fondo.
   *
   *@returns color de fondo.   
   *@see #SetBackgroundColor
   */
  bicotiPixelRGBA GetBackgroundColor();

  /** 
   * Fija color principal.
   *
   *@param color principal.
   */
  void SetForegroundColor( bicotiPixelRGBA color);
 
  /** 
   * Devuelve el color principal.
   *
   *@returns color principal.
   */ 
  bicotiPixelRGBA GetForegroundColor();

  /** 
   * Hace visible o invisible a un actor
   *
   * SetVisible( true ) hace visible al actor (se ve en pantalla). SetVisible( false ) hace el actor invisible. Al hacer invisible un actor todos sus hijos se hacen invisibles. Al hacer un actor visible todos sus hijos se hacen visibles, salvo los que que ya estaban invisibles.
   */
  void SetVisible( bool visible );
  
  /**
   * Devuelve si un actor es visible o no.
   *
   *@returns true si el actor es visible, false en caso contrario.
   *@see #SetVisible
   */
  bool GetVisible();
  
  /**
   * Convierte al actor en actor principal.
   *
   * El actor principal controla el event loop. Al cerrar el actor finaliza el vent loop.
   *@param ma true o false. True convierte al actor en actor principal. 
   */
  void SetMainActor( bool ma );
  
  /**
   * Indica si el actor es actor principal.
   *
   *@returns true si el actor es actor principal, false en caso contrario.
   *@see #SetMainActor
   */
  bool GetMainActor();
  
  /** 
   * Indica si est habilitado para seguir el movimiento del ratn.
   *
   *@returns  true si el actor est habilitado, false en caso contrario.
   */
  bool IsTrackingOn();

 /**
  * Simula la opresin de un botn del ratn.
  *
  *@param x coordenada horizontal del ratn.
  *@param y coordenada vertical del ratn.
  *@param mask botn oprimido (derecho, medio izquierdo).
  *@see button_cases
  */
  void MousePress( int x, int y, char mask );
  
  /**
  * Simula la liberacin de un botn del ratn.
  *
  *@param x coordenada horizontal del ratn.
  *@param y coordenada vertical del ratn.
  *@param mask botn oprimido (derecho, medio izquierdo).
  *@param mask botn liberado (derecho, medio, izquierdo).
  *@see button_cases
 */
  void MouseRelease( int x, int y, char mask );

  /** 
   * Fija la fuente del actor.
   *
   *@param font fuente. 
   *@see bicotiFont
   */
  void SetFont( bicotiFont font );
  
  /**
   * Devuelve la fuente del actor.
   *
   *@returns fuente del actor.
   *@see #SetFont
   *@see bicotiFont
   */
  bicotiFont GetFont();
};


//---------------bicotiHomologousGraphic---------------

/**
 * Homlogo para bicotiActorGraphic2D.
 *
 *@see bicotiActorGraphic2D
 */
class bicotiHomologousGraphic : public bicotiHomologous
{ 
public:

  /** Destructor */
  virtual ~bicotiHomologousGraphic();
  
/**
   * Agrega un hijo al homlogo.
   *
   * SetChild agrega el homlogo indicado.
   *@param ptr_child homlogo hijo.
   */
  virtual void SetChild( bicotiHomologousGraphic * ptr_child )=0;

 /** 
   * Remueve un hijo del homlogo.
   *
   * Remove child quita al homlogo indicado. 
   *@param ptr_child homlogo hijo.
   */  
virtual void RemoveChild( bicotiHomologousGraphic * ptr_child )=0;

/** 
   * Hace visible o invisible a un homlogo.
   *
   * SetVisible( true ) hace el homlogo visible (se ve en pantalla). SetVisible( false ) hace el homlogo invisible. Al hacer invisible un homlogo todos sus hijos se hacen invisibles. Al hacer un homlogo visible todos sus hijos se hacen visibles, salvo que fueran invisibles anteriormente.
   */
  virtual void SetVisible( bool visible )=0;

  /**
   * Fija el color de fondo.
   *
   *@param color color de fondo.
   */
  virtual void SetBackgroundColor( bicotiPixelRGBA color )=0;
 
 /** 
   * Fija color principal.
   *
   *@param color principal.
   */ 
virtual void SetForegroundColor( bicotiPixelRGBA color )=0;
  /**
  * Simula la opresin de un botn del ratn.
  *
  *@param x coordenada horizontal del ratn.
  *@param y coordenada vertical del ratn.
  *@param mask botn oprimido (derecho, medio izquierdo).
  *@see button_cases
  */
  virtual void MousePress( int x, int y, char mask )=0;

 /**
  * Simula la liberacin de un botn del ratn.
  *
  *@param x coordenada horizontal del ratn.
  *@param y coordenada vertical del ratn.
  *@param mask botn oprimido (derecho, medio izquierdo).
  *@param mask botn liberado (derecho, medio, izquierdo).
  *@see button_cases
 */
  virtual void MouseRelease( int x, int y, char mask )=0;

/**
   * Activa eventos de movimiento de ratn.
   *
   * Para uso interno. 
   *@see #DeactivateMouseMotionTracking
   */
  virtual void ActivateMouseMotionTracking()=0;
 
 
  /**
   * Desactiva eventos de movimiento de ratn.
   *
   * Para uso interno. Desactiva los eventos de movimientos de ratn. Esto permite mayor eficiencia al no tener que procesar dichos eventos cuando no se desea.
   *@see #ActivateMouseMotionTracking
   */
 virtual void DeactivateMouseMotionTracking()=0;


  /** 
   * Fija la fuente del homlogo.
   *
   *@param font fuente. 
   *@see bicotiFont
   */
  virtual void SetFont( bicotiFont font )=0;

  /**
   * Para uso interno.
   *
   * Se implementa en el homlogo concreto.
   */
 virtual void LoadConcreteInstance()=0;
};



#endif

