#ifndef _bicoti_actor_graphic_2D_scroll_bar
#define _bicoti_actor_graphic_2D_scroll_bar

#include <source/actor/actor_graphic_2D.hpp>
#include <source/interaction/interaction_scroll_bar.hpp>

//---------------bicotiActorGraphic2DScrollBar---------------

/**
 * Esta clase proporciona un "scroll bar"
 *
 * El deslizador del scroll bar tiene asociado un valor que se mueve en un rango de enteros junto a los desplazamientos del mismo.
 * Como todo actor grfico 2D se puede fijar tamao y posicin, adicionalmente peude indicarse si su disposicin es horizontal o vertical. Puede ser hijo de otro actor grfico (por ejemplo un bicotiActorGraphic2DPanel). Es fuente de eventos de "scroll bar". Este actor no se ajusta automaticamente al tamao de su actor padre, el usuario debe proveer el cdigo necesario para que esto suceda. 
 *@see bicotiHomologousGraphic2DScrollBar
 */

class bicotiActorGraphic2DScrollBar : public bicotiActorGraphic2D , public bicotiInteractionSourceScrollBar
{
private:
  int min_val, max_val, current_val, line_step, page_step;
  bool vertical;

public:
  
  /**
   * Constructor 
   *
   * Construye un nuevo bicotiActorGraphic2DScrollBar
   *@param char * name nombre del actor. 
   */
  bicotiActorGraphic2DScrollBar( char * name = NULL );
  
  /** Destructor */
~bicotiActorGraphic2DScrollBar();

  /**
   * Fija el menor valor del rango
   *
   *@param min menor valor del rango
   */
  void SetMinimum( int min );
 
 /**
   * Fija el mayor valor del rango
   *
   *@param max mayor valor del rango
   */
  void SetMaximum( int max );

  /** 
   * Fija el rango dando sus valores extremos
   * 
   *@param min menor valor del rango
   *@param max mayor valor del rango
   */
  void SetRange( int min , int max );
  

  /**
   * Fija el incremento del valor simple ( cuando se oprimen las flechas del "scroll bar" )
   * 
   *@param ls incremento del valor 
   */

  void SetLineStep( int ls );
 
/**
   * Fija el incremento del valor especial del "scroll bar"
   * 
   *@param ps incremento del valor 
   */
  void SetPageStep( int ps );

  /** 
   * Fija el valor del "scroll bar"
   *
   * Se puede usar para fijar un valor inicial para el "scroll bar"
   *@param val valor del "scroll bar"
   */
  void SetValue( int val );
  
  /**
   * Establece si el  "scroll bar" es horizontal o vertical
   *
   *@param vert si vert es true el "scroll bar" es vertical si es false es horizontal
   */
  void SetVertical( bool vert);

  /**
   * Devuelve el menor valor del rango
   *
   *@returns menor valor del rango
   *@see #SetMinimum
   */
  int GetMinimum();

  /**
   * Devuelve el mayor  valor del rango
   *
   *@returns mayor valor del rango
   *@see #SetMaximum
   */
  int GetMaximum();

  /**
   * Devuelve el incremeto simple del valor
   *
   *@returns incremento simple del valor
   *@see #SetLineStep
   */
  int GetLineStep();
  
 /**
   * Devuelve el incremeto especial del valor
   *
   *@returns incremento especial del valor
   *@see #SetPageStep
   */
  int GetPageStep();

  /**
   * Devuelve el valor actual del "scroll bar"
   *
   *@returns valor actual del "scroll bar"
   *@see #SetValue
   */
  int GetValue();

  /** 
   * Indica si el "scroll bar" es vertica u horizontal
   *
   *@returns true si es vertical y false si es horizontal
   *@see #SetVertical
   */
  bool GetVertical();

  /** 
   * Verifica si un valor dado es distinto al valor del "scroll bar"
   *
   * Para uso interno, no se aconseja su uso por parte del usuario. Implementa el mtodo virtual de bicotiInteractionSourceScrollBar. Verifica si el valor dado es distinto del valor del "scroll bar" y en ese caso lo fija como nuevo valor.
   *
   *@param val cualquier valor dentro del rango 
   *@returns true si val es distinto al valor del "scroll bar" o false en caso contrario
   *@see bicotiInteractionSourceScrollBar::CheckScrollBarValue
   */   
 bool CheckScrollBarValue( int val );
  
  /**
   * Verifica si el rango dado es distinto del rango del "scroll bar"
   * 
   * Para uso interno, no se aconseja su uso por parte del usuario. Implementa el mtodo virtual de bicotiInteractionSourceScrollBar.Verifica si el rango dado es distinto del rango del "scroll bar" y en ese caso lo fija como el nuevo rango.
   *@param min menor valor del rango dado
   *@param max mayor valor del rango dado 
   *@returns true si el rango dado es distinto del rango del "scroll bar"
   *@see  bicotiInteractionSourceScrollBar::CheckScrollBarRange
   */ 
  bool CheckScrollBarRange( int min, int max );

 /** 
   * Verifica si cualquiera de los incrementos dados es distinto al incremento correspondiente del "scroll bar"
   *
   * Para uso interno, no se aconseja su uso por parte del usuario. Implementa el mtodo virtual de bicotiInteractionSourceScrollBar. Verifica si cualquiera de los incrementos dados es distinto al incremento correspondiente del "scroll bar" y en ese caso lo fija como el nuevo incremento
   *@param ls incremento simple dado
   *@param ps incremento especial dado
   *@returns true si cualquiera de los incrementos dados es distinto al correspondiente del "scroll bar"
   *@see bicotiInteractionSourceScrollBar::CheckScrollBarStep
   */   
  bool CheckScrollBarStep( int ls , int ps );
};


//-----------bicotiHomologousGraphic2DScrollBar----------

/**
 * Homologo del bicotiActorGraphic2DScrollBar
 *
 *@see bicotiActorGraphic2DScrollBar
 */
class bicotiHomologousGraphic2DScrollBar : public bicotiHomologousGraphic2D
{
public:
  
  /** destructor */ 
  virtual ~bicotiHomologousGraphic2DScrollBar();

  /**
   * Fija el menor valor del rango
   *
   *@param min menor valor del rango
   *@see bicotiActorGraphic2DScrollBar::SetMinimum
   */   
 virtual void SetMinimum( int min )=0;

 /**
   * Fija el mayor valor del rango
   *
   *@param max mayor valor del rango
   *@see bicotiActorGraphic2DScrollBar::SetMaximum
 */
  virtual void SetMaximum( int max )=0;

  /** 
   * Fija el rango dando sus valores extremos
   * 
   *@param min menor valor del rango
   *@param max mayor valor del rango
   *@see bicotiActorGraphic2DScrollBar::SetRange
 */
  virtual void SetRange( int min, int max )=0;

  /**
   * Fija el incremento del valor simple del "scroll bar"
   * 
   *@param ls incremento del valor 
   *@see bicotiActorGraphic2DScrollBar::SetLineStep
 */
  virtual void SetLineStep( int ls )=0;
 
  /**
   * Fija el incremento del valor especial del "scroll bar"
   * 
   *@param ps incremento del valor 
   *@see bicotiActorGraphic2DScrollBar::SetPageStep
   */
  virtual void SetPageStep( int ps )=0;
  
   /** 
   * Fija el valor del "scroll bar"
   *
   * Se puede usar para fijar un valor inicial para el "scroll bar"
   *@param val valor del "scroll bar"
    *@see bicotiActorGraphic2DScrollBar::SetValue
    */ 
  virtual void SetValue( int val )=0;

 /**
   * Establece si el  "scroll bar" es horizontal o vertical
   *
   *@param vert si vert es true el "scroll bar" ser vertical si es false ser horizontal
   *@see bicotiActorGraphic2DScrollBar::SetVertical
   */
  virtual void SetVertical( bool vert )=0;
};

#endif
