#ifndef _bicoti_actor_graphic_menu
#define _bicoti_actor_graphic_menu


#include <source/actor/actor_graphic.hpp>
#include <source/interaction/interaction_menu.hpp>


//--------------------DECLARATIONS---------------------
//----------------bicotiActorGraphicMenu-----------------

/**@#-*/
class bicotiActorGraphicMenuPopup;
/**@#+*/

/**
 * Clase base de la que se derivan los mens. 
 *
 * Un bicotiActorGraphicMenu define las interfaces comunes de los mens soportados por bicoti: bicotiActorGraphicMenuBar y bicotiActorGraphicMenuPopup. Es fuente de eventos de men. Un bicotiActorGraphicMenu puede contener dos tipos de elementos tems y separadores. A su vez un tem puede tener asociado un bicotiActorGraphicMenuPopup.
 *@see bicotiActorGraphicMenuBar
 *@see bicotiActorGraphicMenuPopup
 *@see bicotiHomologousGraphicMenu
 */
class bicotiActorGraphicMenu : public bicotiActorGraphic, public bicotiInteractionSourceMenu
{
protected:
 
  /**
   * Clase para uso interno de bicotiActorGraphicMenu.
   * 
   *InternalStructure es una clase para uso interno de bicotiActorGraphicMenu. Representa los tems y separadores del men.
    *@see bicotiActorGraphicMenu
   */
 class InternalStructure
  {
  public:
    InternalStructure( int id, bool = true, char * = NULL, bicotiActorGraphicMenuPopup * = NULL );
    
    bool operator==( const InternalStructure& ) const;
    
    int id;
    
    bool is_separator;
    char * item_name;
    bicotiActorGraphicMenuPopup * ptr_popup;
  };

  static int current_id;  
  vector< InternalStructure > elements_list;
  
  
  /**
   * Verifica que la opcin pertenzca al men.
   *
   *@param item nombre de la opcin. 
   *@returns true si item est en el men o false en caso contrario.
   */ 
  bool CheckItem( char * item );
  
public:

  /**
   * Constructor.
   *
   *@param act_name nombre del actor.
   */
  bicotiActorGraphicMenu( char * act_name);
  
  /** Destructor */
  virtual ~bicotiActorGraphicMenu();
 
  /**
   * Agrega un tem al men.
   * 
   *@param item nombre de la nueva opcin del men
   *@returns identificador de tem (un entero nico).
   */ 
  int AddItem( char * item );
  
  /**
   * Agrega un item y le asocia un submen. 
   *
   * El submen se desplega automticamente cuando se selecciona ese tem
   *@param item nombre de la nueva opcin del men.
   *@param pupm submen.
   *@returns  identificador del tem (un entero nico). 
  */
  int AddItem( char *item, bicotiActorGraphicMenuPopup * pupm);
 
 /**
  * Agrega un separador al men.
  *
  *Un separdador es una divisin entre las opciones de un men, si se clickea sobre la misma no ocurre nada.
  *@returns identificador del separador (un entero nico).
  */
  int AddSeparator();

  /**
   * Indica si hay un tem  con este identificador en el men.
   * 
   *@param item_id identificador del tem o separador a eliminar.
   *@returns true si item_id es identificador de un tem del men. 
   */
  bool CheckItem( int item_id);

  /**
   * Borra un tem o separador del men. 
   *
   * RemoveItem borra un item o separador dado un identificador. Los identifacadores no se vuelven a usar si se borra el elemento correspondiente. Si no hay un elemento asociado al identificador se produce una excepcin.
   *@param item_id identificador de item a borrar.
   *@exception bicotiException si item_id no es un item del men.
   */
  void RemoveElement( int item_id );
  
  /**
   * Indica si es un tem del men.
   *
   * Dado un identificador indica si el mismo corresponde a un tem. Si no hay un elemento asociado al identificador se produce una excepcin. 
   *@param item_id identificador de tem.
   *@returns true si item_id corresponde a un tem del men.
   *@exception bicotiException si item_id no es un item del men.
   */
  bool IsItem( int item_id);
  
  
  /**
   * Indica si es un separador del men.
   *
   * Dado un identificador indica si el mismo corresponde a un separador. Si no hay un elemento asociado al identificador se produce una excepcin.
   *@param item_id identificador de tem.
   *@returns true si item_id corresponde a un separador del men.
   *@exception bicotiException si item_id no es un item del men.
   */
  bool IsSeparator( int item_id );
  
  /**
   * Devuelve el nombre de la opcin asociada al identificador.
   *
   * Devuelve el nombre de la opcin asociada al identificador. Si el elemento no existe o es un separador se produce una excepcin.
   *@param item_id identificador de tem.
   *@returns el nombre de la opcin asociada a item_id.
   *@exception bicotiException si item_id no es un item del men o es separador.
   */
  char * GetItem( int item_id );
  
  /**
   * Devuelve un puntero al submen asociado al identificador.
   * 
   * Devuelve un puntero al submen asociado al identificador. Si no hay submense devuelve NULL. Si el elemento no existe o es un separador se produce una excepcin.
   *@param item_id identificador de tem.
   *@returns el submen asociado a item_id.
   *@exception bicotiException si item_id no es un item del men o es separador.
   */
  bicotiActorGraphicMenuPopup * GetPopupForItem( int item_id );
 
  /** 
   * Cantidad de elementos del men creados.
   *
   * GetElementsNumber devuelve el valor del identificador asignado al ltimo elemento creado. Ese valor siempre es creciente. 
   *@returns  cantidad de elementos del men creados.
   */
  int GetElementsNumber();
};


//-----------------bicotiHomologousMenu------------------

/**
 * Homlogo del bicotiActorGraphicMenu.
 *
 *@see bicotiActorGraphicMenu
 */

  class bicotiHomologousGraphicMenu : public bicotiHomologousGraphic
  {
  public:

    /** Destructor */
    virtual ~bicotiHomologousGraphicMenu();

    /** 
     *Agrega un tem. 
     *
     *@param item_id identificador de tem.
     *@param item opcin del men.
     *@see bicotiActorGraphicMenu::AddItem
     */
    virtual void AddItem( int item_id, char * item)=0;

    /**
     *Agrega un submen.
     *
     *@param item_id identificador de tem.
     *@param item opcin del men.
     *@param mpup submen desplegable.
     *@see bicotiActorGraphicMenu::AddItem
     */ 
    virtual void AddItem( int item_id, char * item, bicotiActorGraphicMenuPopup * mpup)=0;
   
    /**
     *Agrega un separador.
     *
     *@see bicotiActorGraphicMenu::AddSeparator
     */
    virtual void AddSeparator()=0;
    
    /**
     * Borra un elemento del men.
     *
     *@param item_id identificador de elemento.
     *@see bicotiActorGraphicMenu::AddSeparator
     */
    virtual void RemoveElement( int item_id)=0;
  };
  


#endif
