#ifndef _bicoti_mapper
#define _bicoti_mapper

#include <source/mapper/mapper_jpeg_ijg.cpp>
#include <source/mapper/mapper_tiff_sgi.cpp>



//-----------------------DECLARATIONS---------------------------
//----------------bicotiMapperFacade---------------------


/**
 * Fachada para mapper
 *
 * La fachada simplifica el uso de los mappers. Esta clase es base de las dems fachadas de mapper.
 *@see bicotiMapperFacade2D
 */
template< class PixelType >
class bicotiMapperFacade
{
protected:
  bicotiImageImplementation< PixelType > * ptr_image;
  bicotiMapper< PixelType > * ptr_mapper;
  bicotiDevice * ptr_device;

  void ResetMe();

public:
  /** Destructor */
  virtual ~bicotiMapperFacade(){};
  
  /** 
   * Crea un device file
   *
   * Crea un device file y lo asigna a un puntero interno.
   *@param f_name nombre del archivo
   */
  void BuildDeviceFile( char * f_name);
  
  /**
   * Lee una imagen
   *
   * Lee una imagen del device y la guarda en la bicotiImageImplementation
   *@see #SetImplementation
   *@see #BuildDeviceFile
   */
  void Read();
  
  /**
   * Escribe una imagen
   *
   * Escribe una imagen en el device file a partir de la bicotiImageImplementation
   *@see #SetImplementation
   *@see #BuildDeviceFile
   */ 
  void Write();

  /** 
   * Borra el device
   *
   * Libera la memoria asignada al device y pone el puntero en NULL;
   *@see #BuildDeviceFile
   */  
  void DestroyDevice();
  
  /**
   * Borra el mapper 
   *
   * Libera la memoria asignada al mapper y pone el puntero en NULL;
   */
  void DestroyMapper();
  
  
  /**
   * Borra el mapper y el device
   *
   * Equivale a hacer un DestroyDevice y un DestroyMapper
   */
  void DestroyMe();

  /**
   * Determina la imagen bicoti
   *
   *@param ptr_ima imgen bicoti
   */
  void SetImplementation( bicotiImageImplementation< PixelType > * ptr_ima );

  /**
   * Devuelve la imagen bicoti
   *
   *@returns implementacin de imagen bicoti
   *@see #SetImplementation
   */
  bicotiImageImplementation< PixelType > * GetImplementation();
};



//--------------bicotiMapperFacade2D---------------------

/**
 * Fachada para mapper para imagen 2D
 *
 * Al da de hoy se soportan imagenes TIFF y JPEG
 */ 
template< class PixelType , class PixelConverter >
class bicotiMapperFacade2D : public bicotiMapperFacade< PixelType >
{
public:
  bicotiMapperFacade2D( bicotiImageImplementation2DSimple< PixelType > * = NULL );
  ~bicotiMapperFacade2D();

  /**
   * Construye un mapper para leer imgenes JPEG con la libjpeg
   *
   *@see bicotiMapperFacade::DestroyMapper
   *@see bicotiMapperJPEGIJG
   */
  void BuildMapperJPEGIJG();
 
 /**
  * Construye un mapper para leer imgenes TIFF con la libtiff
  *
  *@see bicotiMapperFacade::DestroyMapper
  *@see bicotiMapperTIFFSGI
  */
  void BuildMapperTIFFSGI();
};


//---------------------IMPLEMENTATIONS---------------------------
//--------------bicotiMapperFacade-----------------------
template< class PixelType >
void bicotiMapperFacade< PixelType > :: ResetMe()
{
  if ( ptr_device != NULL )
    delete ptr_device;
  
  if ( ptr_mapper != NULL )
    delete ptr_mapper;

  ptr_device = NULL;
  ptr_mapper = NULL; 
};


template< class PixelType >
void bicotiMapperFacade< PixelType > :: BuildDeviceFile( char * f_name )
{
  if( ptr_device != NULL )
    delete ptr_device;

  ptr_device = new bicotiDeviceFile( f_name );
};


template< class PixelType >
void bicotiMapperFacade< PixelType > :: Read()
{
  if( ( ptr_device == NULL ) || ( ptr_mapper == NULL ) || ( ptr_image == NULL ) )
    throw bicotiException( "bicotiMapperFacade", "Read()", 6 );
  
  ptr_mapper->SetDevice( ptr_device );
  ptr_mapper->SetImplementation( ptr_image );
  ptr_mapper->Read();
};


template< class PixelType >
void bicotiMapperFacade< PixelType > :: Write()
{
  if( ( ptr_device == NULL ) || ( ptr_mapper == NULL ) || ( ptr_image == NULL ) )
    throw bicotiException( "bicotiMapperFacade", "Write()", 6 );
  
  ptr_mapper->SetDevice( ptr_device );
  ptr_mapper->SetImplementation( ptr_image );
  ptr_mapper->Write();
};


template< class PixelType >
void bicotiMapperFacade< PixelType > :: DestroyDevice()
{
  if ( ptr_device != NULL )
    delete ptr_device;
  
  ptr_device = NULL;
}


template< class PixelType >
void bicotiMapperFacade< PixelType > :: DestroyMapper()
{
  if ( ptr_mapper != NULL )
    delete ptr_mapper;
  
  ptr_mapper = NULL;
}


template< class PixelType >
void bicotiMapperFacade< PixelType > :: DestroyMe()
{
  DestroyDevice();
  DestroyMapper();
}

   
template< class PixelType >
bicotiImageImplementation< PixelType > * bicotiMapperFacade< PixelType > :: GetImplementation()
{
  return ptr_image;
}


template< class PixelType >
void bicotiMapperFacade< PixelType > :: SetImplementation( bicotiImageImplementation< PixelType > * ptr_ima )
{
  ptr_image = ptr_ima;
}




//---------------bicotiMapperFacade2D--------------------
template< class PixelType, class PixelConverter >
bicotiMapperFacade2D< PixelType, PixelConverter > :: bicotiMapperFacade2D( bicotiImageImplementation2DSimple< PixelType > * ptr_i )
{
  ptr_device = NULL;
  ptr_mapper = NULL;

  if( ptr_i != NULL )
    ptr_image = ptr_i;
  else
    ptr_image = NULL;
}


template< class PixelType, class PixelConverter >
bicotiMapperFacade2D< PixelType, PixelConverter > :: ~bicotiMapperFacade2D()
{
  ResetMe();
}


template< class PixelType, class PixelConverter >
void bicotiMapperFacade2D< PixelType, PixelConverter > :: BuildMapperJPEGIJG()
{
  if( ptr_mapper != NULL )
    delete ptr_mapper;
  
  ptr_mapper = new bicotiMapperJPEGIJG< PixelType, PixelConverter >();
}


template< class PixelType, class PixelConverter >
void bicotiMapperFacade2D< PixelType, PixelConverter > :: BuildMapperTIFFSGI()
{
  if( ptr_mapper != NULL )
    delete ptr_mapper;
  
  ptr_mapper = new bicotiMapperTIFFSGI< PixelType, PixelConverter >();
}




#endif
