//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _numerical_algorithms
#define _numerical_algorithms



#include <source/math_algorithms/math_algorithms_basic.hpp>



//***********************DECLARATIONS*****************************
//----------------------bicotiFunction----------------------------
template< class CodomainType >
class bicotiFunction
{
public:
  // virtual ~bicotiFunction(){};
  virtual CodomainType operator() ( double ) = 0;
};



//----------------------bicotiFunction----------------------------
template< class CodomainType >
class bicotiNumericalAlgorithms
{
public:
  double Bipartition( bicotiFunction< double > *, double, double, double = BIPART_DOM_ERR_TOL, double = BIPART_COD_ERR_TOL, unsigned = BIPART_ITER_MAX );
  double Secant( bicotiFunction< double > *, double, double, double = SECANT_DOM_ERR_TOL, double = SECANT_COD_ERR_TOL, unsigned = SECANT_ITER_MAX );
  double Newton( bicotiFunction< double > *, bicotiFunction< double > *, double, double = NEWTON_DOM_ERR_TOL, double = NEWTON_COD_ERR_TOL, unsigned = NEWTON_ITER_MAX );
  double Quadrature( bicotiFunction< double > *, double, double, double = QUAD_ERR_TOL );
  CodomainType Derivate( bicotiFunction< CodomainType > *, double, double = DERIVATE_ERR_TOL );
  CodomainType Derivate2( bicotiFunction< CodomainType > *, double, double = DERIVATE2_ERR_TOL );
};




//*********************IMPLEMENTATIONS****************************
//----------------------bicotiFunction----------------------------
template< class CodomainType >
double bicotiNumericalAlgorithms< CodomainType > :: Bipartition( bicotiFunction< double > * ptr_fun, double x_first, double x_last, double dom_err_tol, double cod_err_tol, unsigned iter_max )
{
  double xa = x_first;
  double xb = x_last;
  double xm = ( xa + xb ) / 2;

  double ya = (*ptr_fun)( xa );
  double yb = (*ptr_fun)( xb );
  double ym = (*ptr_fun)( xm );

  unsigned iter = 0;

  if( xa > xb )
    throw bicotiException( "bicotiNumericalAlgorithms", "Bipartition", 4 );
  
  while( ( (double)fabs( xb - xa ) > ( ( 1 + (double)fabs( xm ) ) * dom_err_tol ) ) && ( (double)fabs( ym ) > cod_err_tol ) && ( iter < iter_max ) )
    {if ( ( ym * ya ) < 0 )
      {xb = xm;
      yb = (*ptr_fun)( xb );}
    else
      {xa = xm;
      ya = (*ptr_fun)( xa );}
    xm = ( xa + xb ) / 2;
    ym = (*ptr_fun)( xm );
    iter++;}
  
  if ( ( ym * ya ) < 0 )
    return( ( xa + xm ) / 2 );
  else
    {if ( (ym * ya ) > 0)
      return( ( xm + xb ) / 2 );
    else
      return( xm );}
};


template< class CodomainType >
double bicotiNumericalAlgorithms< CodomainType > :: Secant( bicotiFunction< double > * ptr_fun, double x_0, double x_1, double dom_err_tol, double cod_err_tol, unsigned iter_max )
{
  double xn_1 = x_0;
  double xn = x_1;

  unsigned iter = 0;  

  double yn_1 = (*ptr_fun)( xn_1 );
  double yn = (*ptr_fun)( xn );

  double xn1 = xn - yn * ( ( xn - xn_1 ) / ( yn - yn_1 ) );
  double yn1 = (*ptr_fun)( xn1 );
    
  while( ( 2 * fabs( xn1 - xn ) > ( ( 1 + (double)fabs( xn1 ) ) * dom_err_tol ) ) && ( fabs( yn1 ) > cod_err_tol ) && ( iter < iter_max ) )
    {xn_1 = xn;
    yn_1 = yn;
    xn = xn1;
    yn = yn1;
    xn1 = xn - yn * ( ( xn - xn_1 ) / ( yn - yn_1 ) );
    yn1 = (*ptr_fun)( xn1 );
    iter++;}

  return( xn1 );
};


template< class CodomainType >
double bicotiNumericalAlgorithms< CodomainType > :: Newton( bicotiFunction< double > * ptr_fun, bicotiFunction< double > * ptr_fun_der, double x_seed, double dom_err_tol, double cod_err_tol, unsigned iter_max )
{
  double xn = x_seed;
  
  unsigned iter = 0;  
  
  double yn = (*ptr_fun)( xn );
  double dyn = (*ptr_fun_der)( xn );
  
  double xn1 = xn - yn / dyn;
  
  while( ( 2 * fabs( xn1 - xn ) > ( ( 1 + (double)fabs( xn1 ) ) * dom_err_tol ) ) && ( fabs( yn ) > cod_err_tol ) && ( iter < iter_max ) )
    {xn = xn1;
    yn = (*ptr_fun)( xn );
    dyn = (*ptr_fun_der)( xn );
    xn1 = xn - yn / dyn;
    iter++;}

  return( xn1 );
};


template< class CodomainType >
double bicotiNumericalAlgorithms< CodomainType > :: Quadrature( bicotiFunction< double > * ptr_fun, double x_first, double x_last, double quad_err_tol )
{
  unsigned n, m, i, num = 1, diag = 3;
  double error, two_exp_2m;
  vector< vector< double > > quads;
  vector< double > aux_vector;
  
  double a = x_first;
  double b = x_last;
  double h = ( b - a );
  double quad = 0.5 * ( (*ptr_fun)( a ) + (*ptr_fun)( b ) ) * h;
  double quad_aux = 0;
  quads.push_back( aux_vector );
  quads[ 0 ].push_back( quad );
  
  do
    {m = 1;
    n = diag - m;
    two_exp_2m = 4;
    
    quad_aux = 0;
    h = 0.5 * h;
    for( i = 0; i < num; i++ )
      quad_aux += (*ptr_fun)( a + h * ( 2 * i + 1 ) );
    quad = 0.5 * quad + h * quad_aux;
    num *= 2;
    quads[ 0 ].push_back( quad );
    
    while( n != 1 )
      {if( n == 2 )
	quads.push_back( aux_vector );
      quads[ m ].push_back( ( quads[ m - 1 ][ n - 2 ] - two_exp_2m * quads[ m - 1 ][ n - 1 ] ) / ( 1 - two_exp_2m ) );
      m++;
      n--;
      two_exp_2m *= 4;}

    error = (double)fabs( quads[ m - 2 ][ 0 ] - quads[ m - 2 ][ 1 ] ) / ( two_exp_2m - 1 );
    diag++;} while ( ( error / ( 1 + (double)fabs(quads[ m - 1 ][ 0 ]) ) ) > quad_err_tol );

  return( quads[ m - 1 ][ 0 ] );
};


template< class CodomainType >
CodomainType bicotiNumericalAlgorithms< CodomainType > :: Derivate( bicotiFunction< CodomainType > * ptr_fun, double x_point, double deriv_err_tol )
{
  unsigned n, m, diag = 3;
  double error, two_exp_2m;
  vector< vector< CodomainType > > derivates;
  vector< CodomainType > aux_vector;
  CodomainType cod_aux;
  
  double h = 10 * EPSILON_MACHINE;
  CodomainType derivate = ( (*ptr_fun)( x_point + h ) - (*ptr_fun)( x_point - h ) ) * ( 1 / ( 2 * h ) );
  derivates.push_back( aux_vector );
  derivates[ 0 ].push_back( derivate );
  
  do
    {m = 1;
    n = diag - m;
    two_exp_2m = 4;
    
    h = 0.5 * h;
    derivate = ( (*ptr_fun)( x_point + h ) - (*ptr_fun)( x_point - h ) ) * ( 1 / ( 2 * h ) );
    derivates[ 0 ].push_back( derivate );
    
    while( n != 1 )
      {if( n == 2 )
	derivates.push_back( aux_vector );
      derivates[ m ].push_back( ( derivates[ m - 1 ][ n - 2 ] - derivates[ m - 1 ][ n - 1 ] * two_exp_2m ) * ( 1 / ( 1 - two_exp_2m ) ) );
      m++;
      n--;
      two_exp_2m *= 4;}
    
    cod_aux = ( derivates[ m - 2 ][ 0 ] - derivates[ m - 2 ][ 1 ] ) * ( 1 / ( two_exp_2m - 1 ) );
    error = (double)fabs( cod_aux );
    diag++;} while ( ( error / ( 1 + (double)fabs(derivates[ m - 1 ][ 0 ]) ) ) > deriv_err_tol );

  return( derivates[ m - 1 ][ 0 ] ); 
};


template< class CodomainType >
CodomainType bicotiNumericalAlgorithms< CodomainType > :: Derivate2( bicotiFunction< CodomainType > * ptr_fun, double x_point, double deriv2_err_tol )
{
  unsigned n, m, diag = 3;
  double error, two_exp_2m;
  vector< vector< CodomainType > > derivates;
  vector< CodomainType > aux_vector;
  CodomainType cod_aux;
  
  double h = 100 * EPSILON_MACHINE;
  CodomainType derivate = ( (*ptr_fun)( x_point + h ) + (*ptr_fun)( x_point - h ) - (*ptr_fun)( x_point ) * 2 ) * ( 1 / ( h * h ) );
  derivates.push_back( aux_vector );
  derivates[ 0 ].push_back( derivate );
  
  do
    {m = 1;
    n = diag - m;
    two_exp_2m = 4;
    
    h = 0.5 * h;
    derivate = ( (*ptr_fun)( x_point + h ) + (*ptr_fun)( x_point - h ) - (*ptr_fun)( x_point ) * 2 ) * ( 1 / ( h * h ) );
    derivates[ 0 ].push_back( derivate );
    
    while( n != 1 )
      {if( n == 2 )
	derivates.push_back( aux_vector );
      derivates[ m ].push_back( ( derivates[ m - 1 ][ n - 2 ] - derivates[ m - 1 ][ n - 1 ] * two_exp_2m ) * ( 1 / ( 1 - two_exp_2m ) ) );
      m++;
      n--;
      two_exp_2m *= 4;}
    
    cod_aux = ( derivates[ m - 2 ][ 0 ] - derivates[ m - 2 ][ 1 ] ) * ( 1 / ( two_exp_2m - 1 ) );
    error = (double)fabs( cod_aux );
    diag++;} while ( ( error / ( 1 + (double)fabs(derivates[ m - 1 ][ 0 ]) ) ) > deriv2_err_tol );

  return( derivates[ m - 1 ][ 0 ] );
};




#endif 
