#ifndef _histogram_painter
#define _histogram_painter

#include <source/property/property_histogram_basic.hpp>
#include <source/actor/actor_graphic_2D_image.hpp>


//******************************DECLARATION*************************************
//-------------------------bicotiHorizontalStrategy-----------------------------

/**
 * Estrategia para tratar la componente horizontal del histograma ( valores de los pxeles )
 *
 * Clase base para todas las estrategias horizotales
 *@see bicotiHorizontalStrategyLinear
 */ 
template< class ColorType >
class bicotiHorizontalStrategy
{
protected:
  ColorType range;
  int width;

public:
  /**
   * Establece el rango de los pixeles sobre el eje horizontal
   *
   *@param color valor del rango
   */
  void SetRange( ColorType color);

  /**
   * Establece el ancho del eje horizontal
   *
   *@param w ancho del eje horizontal
   */
  void SetTargetWidth( int w );

  /** 
   * Calcula los parmetros internos  
   *
   * Los parmetros se calculan de acuerdo a los datos proporcionados y la estrategia en particular
   */ 
  virtual void ComputeParameters() = 0;
 
  /**
   * Devuelve la absiza adecuada al valor del pixel
   *
   *@param color valor del pixel
   */
  virtual int operator()( ColorType color) = 0;
};


//----------------------bicotiHorizontalStrategyLinear--------------------------

/** 
 * bicotiHorizontalStrategy lineal
 *
 */
template< class ColorType >
class bicotiHorizontalStrategyLinear : public bicotiHorizontalStrategy< ColorType >
{
protected:
  float shape_factor;

public:

  /** Constructor */
  bicotiHorizontalStrategyLinear();
  
  /** Destructor */
  ~bicotiHorizontalStrategyLinear();

  /**
   * Calcula los parmetros internos 
   *
   * Implementa el mtodo virtual de bicotiHorizontalStrategy
   */
  void ComputeParameters();
  
  /**
   * Devuelve la absiza adecuada al valor del pixel
   *
   *@param color valor del pixel 
   */
  int operator()( ColorType color );
};


//--------------------------bicotiVerticalStrategy-----------------------------
/**
 * Estrategia para tratar la componente vertical del histograma ( frecuencia )
 *
 * Clase base para todas las estrategias verticales
 *@see bicotiVerticalStrategyLinear
 */ 
class bicotiVerticalStrategy
{
protected:
  int range;
  int height;
  
public:

  /**
   * Establece el rango des la frecuencias
   *
   *@param ran valor del rango
   */
  void SetRange( int ran );
 
  /**
   * Establece la altura del eje vertical
   *
   *@param altura del eje vertical
   */
  void SetTargetHeight( int h );

 
  /** 
   * Calcula los parmetros internos  
   *
   * Los parmetros se calculan de acuerdo a los datos proporcionados y la estrategia en particaular
   */ 
  virtual void ComputeParameters() = 0;
 
  /**
   * Devuelve la ordenada adecuada a la frecuencia del pixel
   *
   *@param frec frecuencia del pixel
   */
  virtual int operator()( int frec ) = 0;
};


//-----------------------bicotiVerticalStrategyLinear---------------------------

/** 
 * bicotiVerticalStrategy vertical
 *
 */
class bicotiVerticalStrategyLinear : public bicotiVerticalStrategy
{
protected:
  float shape_factor;

public:
  
  /** Constructor */
  bicotiVerticalStrategyLinear();
  
  /** Destructor*/
  ~bicotiVerticalStrategyLinear();

  /**
   * Calcula los parmetros internos 
   *
   * Implementa el mtodo virtual de bicotiVerticalStrategy
   */
  void ComputeParameters();

  
  /**
   * Devuelve la ordenada adecuada a la frecuencia del pixel
   *
   *@param frec frecuencia
   */
  int operator()( int frec );
};


//---------------------------bicotiColorStrategy--------------------------------
template< class ColorType, class PixelTargetType >
class bicotiColorStrategy
{
public:
  virtual PixelTargetType operator()( ColorType, int ) = 0;
};


//-----------------------bicotiColorStrategyConstant----------------------------
template< class ColorType, class PixelTargetType >
class bicotiColorStrategyConstant : public bicotiColorStrategy< ColorType, PixelTargetType >
{
private:
  PixelTargetType pixel;

public:
  void SetPencil( PixelTargetType );
  PixelTargetType operator()( ColorType, int );
};


//---------------------------bicotiHistogramPainter-----------------------------

/**
 * Dibuja un histograma sobre un bicotiActorGraphic2DImage
 *
 */ 
template< class ColorType, class PixelSourceType = ColorType, class PixelTargetType = PixelSourceType >
class bicotiHistogramPainter
{
protected:
  bicotiActorGraphic2DImage< PixelTargetType > * ptr_actor;
  bicotiPropertyHistogram< ColorType, PixelSourceType > * ptr_histo;

  vector< bicotiVerticalStrategy * > vert_strat;
  vector< bicotiHorizontalStrategy< ColorType > * > horiz_strat;
  vector< bicotiColorStrategy< ColorType, PixelTargetType > * > color_strat;
  vector< int > x_0, y_0, x_1, y_1;

  int width, height;

  /**
   * Resetea valores iniciales del bicotiHistogramPainter
   *
   */
  void ResetMe();
  
  /**
   * Hace los ajustes necesarios antes de pintar un histograma
   *
   *@exception bicotiException si no hay imagen o histograma
   *@exception bicotiException si los lmites de las regiones est fuera de la imagen
   *@see #AddLayoutInformation   
   */
  void Preamble();

public:
  typedef bicotiActorGraphic2DImage< PixelTargetType > actor_histogram;
  
  bicotiHistogramPainter();
  ~bicotiHistogramPainter();
  
  /**
   * Establece el histograma 
   *
   *@param ptr_prop histogram a representar
   */
  void SetPropertyHistogram( bicotiPropertyHistogram< ColorType, PixelSourceType > * ptr_prop );
 
  /**
   * Establece la imagen 
   *
   *@param ptr_act imagen sobre la que se dibujar el histograma
   */
  void SetActorImage( bicotiActorGraphic2DImage< PixelTargetType > * ptr_act );


  /**
   * Agrega informacin de como dibujar el histograma
   *
   *@param ptr_vert estrategia vertical
   *@param ptr_horiz estrategia horizontal
   *@param ptr_color estrategia de color
   *@param x0 coordenada horizontal superior de la regin donde se aplican las estrategias 
   *@param y0 coordenada vertical  superior de la regin donde se aplican las estrategias
   *@param x1 coordenada horizontal inferior de la regin donde se aplican las estrategias
   *@param y1 coordenada vertical inferior de la regin donde se aplican las estrategias
   */
  void AddLayoutInformation( bicotiVerticalStrategy * ptr_vert, bicotiHorizontalStrategy< ColorType > * ptr_horiz, bicotiColorStrategy< ColorType, PixelTargetType > * ptr_color, int x0, int y0, int x1, int y1);

  
  /**
   * Pinta los ejes del histograma
   *
   *@param color color de los ejes
   */
  void PaintAxes( PixelTargetType color);
  
  /**
   * Pinta el histograma como una grfica de puntos
   *
   */
  void PaintDot();
  
  /**
   * Pinta el historgrama como una grfica de barras
   *
   */
  void PaintBar();
  
  /** 
   * Pinta el histogrma como una grfica slida
   *
   */
  void PaintSolid();
};


//****************************IMPLEMENTATION************************************
//-------------------------bicotiHorizontalStrategy-----------------------------
template< class ColorType >
void bicotiHorizontalStrategy< ColorType > :: SetRange( ColorType color )
{
  range = color;
}

template< class ColorType >
void bicotiHorizontalStrategy< ColorType > :: SetTargetWidth( int w )
{
  width = w;
}


//----------------------bicotiHorizontalStrategyLinear--------------------------
template< class ColorType >
bicotiHorizontalStrategyLinear< ColorType > :: bicotiHorizontalStrategyLinear()
{
}

template< class ColorType >
bicotiHorizontalStrategyLinear< ColorType > :: ~bicotiHorizontalStrategyLinear()
{
}

template< class ColorType >
void bicotiHorizontalStrategyLinear< ColorType > :: ComputeParameters()
{
  shape_factor = ( (float)width / range );
}

template< class ColorType >
int bicotiHorizontalStrategyLinear< ColorType > :: operator()( ColorType color )
{
  return ((int)( color * shape_factor));
}


//--------------------------bicotiVerticalStrategy------------------------------
void bicotiVerticalStrategy :: SetRange( int ran )
{
  range = ran;
}

void bicotiVerticalStrategy :: SetTargetHeight( int h )
{
  height = h;  
}


//-----------------------bicotiVerticalStrategyLinear---------------------------
bicotiVerticalStrategyLinear :: bicotiVerticalStrategyLinear()
{
}

bicotiVerticalStrategyLinear :: ~bicotiVerticalStrategyLinear()
{
}

void bicotiVerticalStrategyLinear :: ComputeParameters()
{
  shape_factor = ( (float)height / range );
}

int bicotiVerticalStrategyLinear :: operator()( int frec )
{
  return ((int)( frec * shape_factor));
}


//-----------------------bicotiColorStrategyConstant----------------------------
template< class ColorType, class PixelTargetType >
void bicotiColorStrategyConstant< ColorType, PixelTargetType > :: SetPencil( PixelTargetType pix )
{
  pixel = pix;
}


template< class ColorType, class PixelTargetType >
PixelTargetType bicotiColorStrategyConstant< ColorType, PixelTargetType > :: operator()( ColorType color, int frequency )
{
  return pixel;
}


//-------------------------bicotiHistogramPainter------------------------------
template< class ColorType, class PixelSourceType, class PixelTargetType >
bicotiHistogramPainter< ColorType, PixelSourceType, PixelTargetType > :: bicotiHistogramPainter()
{
  ResetMe();
}

template< class ColorType, class PixelSourceType, class PixelTargetType >
bicotiHistogramPainter< ColorType, PixelSourceType, PixelTargetType > :: ~bicotiHistogramPainter()
{
}

template< class ColorType, class PixelSourceType, class PixelTargetType >
void bicotiHistogramPainter< ColorType, PixelSourceType, PixelTargetType > :: ResetMe()
{
  ptr_actor = NULL;
  ptr_histo = NULL;
}


template< class ColorType, class PixelSourceType, class PixelTargetType >
void bicotiHistogramPainter< ColorType, PixelSourceType, PixelTargetType > :: Preamble()
{
  if ( (ptr_actor == NULL) || (ptr_histo == NULL) )
    
   throw bicotiException( "bicotiHistogramPainter" , "Preamble", 6 ); 
  
  width = ptr_actor->GetX0Size();
  height = ptr_actor->GetX1Size();

  ptr_actor->SetSize( width, height );
  
  int max_pix_num;
  for( int i = 0; i < vert_strat.size(); i++ )
    {if ( ( x_0[i] >= 0 ) && ( x_1[i] <= ptr_actor->GetX0Size() ) )
      width = x_1[i] - x_0[i];
    else
      throw bicotiException( "bicotiHistogramPainter" , "Preamble" , 4 ); 
    
    if ( ( y_0[i] >= 0 ) && ( y_1[i] <= ptr_actor->GetX1Size() ) )
      height = y_1[i] - y_0[i];
    else
      throw bicotiException( "bicotiHistogramPainter" , "Preamble" , 4 ); 
    
    horiz_strat[i]->SetTargetWidth( width );
    horiz_strat[i]->ComputeParameters();
    
    max_pix_num = 0;
    for( ptr_histo->First(i); !ptr_histo->IsDone(i); ptr_histo->Next(i) )
      {if ( ptr_histo->GetCurrentPointsNumber(i) > max_pix_num )
	max_pix_num = ptr_histo->GetCurrentPointsNumber(i);}
    
    vert_strat[i]->SetRange( max_pix_num );
    vert_strat[i]->SetTargetHeight( height );
    vert_strat[i]->ComputeParameters();}
}

template< class ColorType, class PixelSourceType, class PixelTargetType >
void bicotiHistogramPainter< ColorType, PixelSourceType, PixelTargetType > :: SetPropertyHistogram( bicotiPropertyHistogram< ColorType, PixelSourceType > * ptr_prop )
{
  ptr_histo = ptr_prop;
}

template< class ColorType, class PixelSourceType, class PixelTargetType >
void bicotiHistogramPainter< ColorType, PixelSourceType, PixelTargetType > :: SetActorImage( bicotiActorGraphic2DImage< PixelTargetType > * ptr_act )
{
  ptr_actor = ptr_act;
}


template< class ColorType, class PixelSourceType, class PixelTargetType >
void bicotiHistogramPainter< ColorType, PixelSourceType, PixelTargetType > :: AddLayoutInformation( bicotiVerticalStrategy * ptr_vert, bicotiHorizontalStrategy< ColorType > * ptr_horiz, bicotiColorStrategy< ColorType, PixelTargetType > * ptr_color, int x0, int y0, int x1, int y1 )
{
  vert_strat.push_back( ptr_vert );
  horiz_strat.push_back( ptr_horiz );
  color_strat.push_back( ptr_color );
  x_0.push_back( x0 );
  y_0.push_back( y0 );
  x_1.push_back( x1 );
  y_1.push_back( y1 );
}


template< class ColorType, class PixelSourceType, class PixelTargetType >
void bicotiHistogramPainter< ColorType, PixelSourceType, PixelTargetType > :: PaintAxes( PixelTargetType color )
{
  Preamble();
  int i;

  for( int j = 0; j < vert_strat.size(); j++ )
    {for( i = x_0[j]; i < x_1[j]; i++ )
      ptr_actor->SetPixel( i, y_1[j], color );
    for( i = y_0[j]; i < y_1[j]; i++ )
      ptr_actor->SetPixel( x_0[j], i, color );}
}


template< class ColorType, class PixelSourceType, class PixelTargetType >
void bicotiHistogramPainter< ColorType, PixelSourceType, PixelTargetType > :: PaintDot()
{
  Preamble();

  ColorType color;
  int frequency;

  for( int i = 0; i < vert_strat.size(); i++ )
    {for( ptr_histo->First(i); !ptr_histo->IsDone(i); ptr_histo->Next(i) )
      {color = ptr_histo->GetCurrentColor(i);
      frequency = ptr_histo->GetCurrentPointsNumber(i);
      ptr_actor->SetPixel( x_0[i] + (*(horiz_strat[i]))( color ), y_1[i] - 1 - (*(vert_strat[i]))( frequency ), (*(color_strat[i]))( color, frequency ) );}}
}


template< class ColorType, class PixelSourceType, class PixelTargetType >
void bicotiHistogramPainter< ColorType, PixelSourceType, PixelTargetType > :: PaintBar()
{
  Preamble();

  ColorType color;
  int frequency;
  int bar_h, i, j;
  PixelTargetType pixel;

  for( int k = 0; k < vert_strat.size(); k++ )
    {for( ptr_histo->First(k); !ptr_histo->IsDone(k); ptr_histo->Next(k) )
    {color = ptr_histo->GetCurrentColor(k);
    frequency = ptr_histo->GetCurrentPointsNumber(k);
    pixel = (*(color_strat[k]))( color, frequency );
    i = (*(horiz_strat[k]))( color );
    bar_h = (*(vert_strat[k]))( frequency );
    for( j = 0; j < bar_h; j++ )
      ptr_actor->SetPixel( x_0[k] + i, y_1[k] - 1 - j, pixel );}}
}


template< class ColorType, class PixelSourceType, class PixelTargetType >
void bicotiHistogramPainter< ColorType, PixelSourceType, PixelTargetType > :: PaintSolid()
{
  Preamble();

  ColorType color;
  int frequency;
  int bar_h, i, i_col, i_old, j;
  PixelTargetType pixel;
  
  for( int k = 0; k < vert_strat.size(); k++ )
    {i_old = 0;
    for( ptr_histo->First(k); !ptr_histo->IsDone(k); ptr_histo->Next(k) )
      {color = ptr_histo->GetCurrentColor(k);
      frequency = ptr_histo->GetCurrentPointsNumber(k);
      pixel = (*(color_strat[k]))( color, frequency );
      i_col = (*(horiz_strat[k]))( color );
      bar_h = (*(vert_strat[k]))( frequency );
      for( i = i_old; i < i_col; i++ )
	for( j = 0; j < bar_h; j++ )
	  ptr_actor->SetPixel( x_0[k] + i, y_1[k] - 1 - j, pixel );
      i_old = i_col;}}
}

#endif
